﻿using Abp.Collections.Extensions;
using Hotline.Snapshot;
using Mapster;
using Newtonsoft.Json.Linq;
using SnapshotWinFormsApp.Application.Dtos;
using SnapshotWinFormsApp.Application.Interfaces;
using SnapshotWinFormsApp.Entities.NewHotline;
using SnapshotWinFormsApp.Entities.OldHotline;
using SnapshotWinFormsApp.Repository;
using SnapshotWinFormsApp.Repository.Interfaces;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Linq.Dynamic.Core.Tokenizer;
using System.Text;
using System.Threading.Tasks;

namespace SnapshotWinFormsApp.Application;

[Description("邀请码记录")]
public class InviteLogApplication : ImportApplicationBase<SSP_InviteLogEntity, InviteCodeRecord, string, OldInviteLogEntity>, IImportApplication
{
    private readonly ITargetRepository<InviteCode> _newInviteCodeRepo;
    private readonly ITargetRepository<InviteCodeRecord> _newInviteLogRepo;
    private readonly ITargetRepository<Citizen> _userInfoRepo;
    private readonly ITargetRepository<ThirdAccount> _thirdAccountRepo;
    private IList<InviteCode> invities;
    private IList<ThirdAccount> thirdAccounts;

    public InviteLogApplication(CreateInstanceInDto inDto) : base(inDto)
    {
        _newInviteCodeRepo = new TargetRepository<InviteCode>(inDto);
        _thirdAccountRepo = new TargetRepository<ThirdAccount>(inDto);
        invities = _newInviteCodeRepo.GetAll();
        thirdAccounts = _thirdAccountRepo.GetAll();
    }

    public override ISugarQueryable<OldInviteLogEntity> GetSourceList(CreateInstanceInDto inDto)
    {
        return _sugarClient.Queryable<SSP_InviteLogEntity>()
            .LeftJoin<SSP_InviteEntity>((log, invite) => log.SSPI_CodeID == invite.SIC_Byte)
            .LeftJoin<WeChatUserEntity>((log, invite, user) => user.WUR_Openid == log.SSPI_Openid)
            .Select((log, invite, user) => new OldInviteLogEntity
            {
                Id = log.Id,
                Name = user.WUR_WebUserName,
                OrgName = invite.SIC_BMName,
                InviteCode = log.SSPI_Code,
                WXOpenId = log.SSPI_Openid,
                PhoneNumber = user.WUR_PhoneNum,
            });
    }

    public override async Task<bool> HasOldDataAsync(string tableName, OldInviteLogEntity item, CancellationToken token)
    {
        return await _newInviteLogRepo.Queryable().AnyAsync(m => m.Id == item.Id, token);
    }

    public override async Task<InviteCodeRecord> GetTargetAsync(OldInviteLogEntity source, CancellationToken token)
    {
        var target = source.Adapt<InviteCodeRecord>();
        target.OrgId = invities.FirstOrDefault(m => m.OrgName == source.OrgName)?.Id ?? string.Empty;
        return await Task.FromResult(target);
    }

    public override async Task InsertAfterAsync(Action<string> log,  OldInviteLogEntity item, InviteCodeRecord target, CancellationToken token)
    {
        var userId = thirdAccounts.Where(m => m.OpenId == item.WXOpenId).Select(m => m.ExternalId).FirstOrDefault();
        if (userId.IsNullOrEmpty()) return;

        await _userInfoRepo.Updateable()
                 .SetColumns(m => m.InvitationCode, item.InviteCode)
                 .Where(m => m.Id == userId)
                 .ExecuteCommandAsync(token);
    }
}
